# ###############################
#
# PolaRX2e.py 
#
# Code made by fjalar@vedur.is 
# Iceland Met Office
# 2011-2012
#
# ###############################

######## IMPORT LIBRARIES ########

import sCurl, re, sys, datetime
import checkPort as checkport
    
######## DEFINE CLASS ########

class PolaRX2e:

    # ALL TEXT HERE IN THE DOC NEEDS TO BE REVISED LATER ON
    """A Receiver class designed for Septentrio PolaRX2e receiver type.
    
    Public functions:
    getConnection       -- Get the status of the connection to the router and receiver.
    getVolt             -- Not implemented.
    getTemp             -- Not implemented.
    getLogging          -- Get the logging status of the receiver.
    getSessions         -- List all sessions defined on the receiver and their status; active or inactive.
    getDirList          -- Not implemented.
    getFileList         -- Get a list of files for a specific directory passed as an argument.
    getPosition         -- Get the current position of the receiver.
    getTracking         -- Get the current tracking of satellites.
    getFirmwareVersion  -- Get the current firmware version for the receier.
    getStatus           -- A collective report for the receiver; logging and session.
    info                -- Reports information on the receiver object.

    Private functions:
    __getStuff          -- The function that combines the similar actions of getTemp, getVolt, getLogging and getSessions.
    __parceFancy        -- Parcing method used by the Public methods to trim and cut the http result and return values in dictionaries instead of a singel string.
    """

    def __init__(self,station_info):
        
        # 1 # Parameters saved and type set to string
        self.station_info = station_info        
        self.sid = self.station_info['station']['id']  
        self.userpass = None
        self.CONNECTION_STATUS = {'router':None,'receiver':None}
        
        # 2 # The router IP here is the ip on the router/receiver connected to a SIL station. IP is on the form 192.168.100.XX
        if self.station_info['proxy']['sshserver'] == '':

            self.ip = station_info['router']['ip']
            self.ipport = self.ip
            self.port = station_info['receiver']['httpport']

        # 3 # If the connection uses a proxy through kjarni the IP is set as localhost to use the ssh-tunnel from the localhost (rek/rek2)
        if station_info['proxy']['sshserver'] != '':         

            self.ip = 'localhost'
            self.ipport = self.ip
            self.port = station_info['proxy']['httpport']

        # 4 # If username and password is required to access the receiver, the user/pass info will be passed to the CURL command in the
            # self.userpass variable (dicitonary)
        if station_info['receiver']['user'] and station_info['receiver']['pwd']:
            self.userpass = {'user':station_info['receiver']['user'],'pass':station_info['receiver']['pwd']}

        # 5 # Set the default value for DEBUG
        self.DEBUG = False
        
        # 6 # Expand IP with port if port is defined
        if self.port:
            self.ipport = '%s:%s'% (self.ip,self.port)
 
        # 8 # Instantiate sCURL object for URL stuff
        self.curl = sCurl.sCurl()
       
        # 8 # Check if station is alive
        self.CONNECTION_STATUS = self.getConnection()
    ## ------------------- getConnection() -------------------##

    def getConnection(self):
        """Fetches connection status with the router and receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc

        
        result ={'router':None,'receiver':None,}

        result = checkport.isOpen(self.ip,self.port,self.DEBUG)

        return result


    ## ------------------- getVolt() -------------------##

    def getVolt(self):
        """Fetches voltage information from receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc


        result = {'value':'N/S','error':''}
        return result 

    ## ------------------- getTemp() -------------------##

    def getTemp(self):
        """Fetches temperature information from receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc


        result = {'value':'N/S','error':''}
        return result 
    ## ------------------- getLogging() -------------------##
    def getLogging(self):
        """Fetches information about the logging status of the receiver; What files are currently being created"""        

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc
        
        result = {'value':None,'error':None}
        fileList = {'value':None,'error':None}

        # 1 # Get the current file list from the receiver
        fileList = self.__getStuff('filelist')
        
        # 2 # Get the currnent date and time: YYYY MM DD
        now = datetime.datetime.now()

        # 2.1 # Format month correctly by adding 0 in front of single digit numbers
        if len(str(now.month))==1:
            month = '0%s' % now.month
        else:
            month = now.month
        
        # 2.2 # Format day correctly by adding 0 in front of single digit numbers
        if len(str(now.day))==1:
            day = '0%s' % now.day
        else:
            day = now.day

        # 2.3 # Format hour correctly by adding 0 in front of single digit numbers
        if len(str(now.hour))==1:
            hour = '0%s' % now.hour
        else:
            hour = now.hour
        # 2.4 # Then put the "safe" rest to proper variables
        year = now.year
        minute = now.minute

        # 2.5 # DEBUG for date-time formats
        if self.DEBUG: print "%s YEAR: %s MONTH:%s DAY:%s HOUR:%s MINUTE:%s  " % (currFunc,now.year,now.month, now.day, now.hour, now.minute)
        if self.DEBUG: print "%s New MONTH:%s New DAY:%s New HOUR:%s " % (currFunc,month, day, hour)
        if self.DEBUG: print "%s fileList value: %s" % (currFunc,fileList['value'])
        #print "File list error:", fileList['error']
        #print "File list value:", fileList['value']

        # 3 # Find the file in the fileList that has the current date and hour right.. which is the file being currently logged.
        if fileList['value']:
            for f in fileList['value']:
                if f['year'] == str(year) and f['month'] == str(month) and f['day'] == str(day) and f['hour'] == str(hour):
                    result['value'] = f
                    if self.DEBUG: print '%s Current logging file is -> Date: %s-%s-%s Time: %s:%s Size:%s Name:%s' % (
                                                                                            currFunc,
                                                                                            f['year'],
                                                                                            f['month'],
                                                                                            f['day'],
                                                                                            f['hour'],
                                                                                            f['minute'],
                                                                                            f['size'],
                                                                                            f['name'])
                
        else:
            result['error'] = "Error in getLogging()"

        if self.DEBUG: "%s result: %s" % (currFunc,result)
        return result

    ## ------------------- getSession() -------------------##

    def getSessions(self):
        """Fetches information about defined sessions from the receiver; which sessions are active and which are not."""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc

        result = self.__getStuff('sessions')
        return result

    def getPosition(self):
        """Fetches information about the position of the receiver.. from the receiver."""
        
        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc

        result = self.__getStuff('position')
        return result        

    def getFileList(self,directory=''):
        """Fetches directory listing from a receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc

        result = self.__getStuff('filelist')
        return result

    def getTracking(self):
        """Fetches tracking status for a receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc

        result = self.__getStuff('tracking')
        return result

    def getFirmwareVersion(self):
        """Fetches the firmware version for the receiver"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc

        result = self.__getStuff('firmwareversion')
        return result

    def getStatus(self):
        """Compiles a status report for the receiver; temp, volt, logging, session and returns as a dictionary """

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc
        
        # 1 # Define local variables 

        report = {'connection':'','temp':'','volt':'','logging':'','session':'','position':''}
        
        # 2 # Do the funky dance..
        report['connection'] = self.getConnection()
        report['temp'] = self.getTemp()
        report['volt'] = self.getVolt()
        report['logging'] = self.getLogging()
        report['session'] = self.getSessions()
        report['position'] = self.getPosition()  
  
        
        return report
        
    def setDebug(self):
        """Debug ON/OFF switch."""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc

        if self.DEBUG:
            self.DEBUG = False
            print 'Debug OFF'
        else:
            self.DEBUG = True
            print 'Debug ON'
        

    def info(self):

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc

        """Information on the object's attributes"""
        print ''
        print 'SID = %s' % self.sid
        print 'IP = %s' % self.ip
        print 'PORT = %s' % self.port
        print 'DEBUG = %s' % self.DEBUG
        print 'ATTRIBUTES =  %s' % dir(self)


    def test(self):
        """Simple method that calls all the public methods in the class without any parameters"""

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc

        test = {'connection':'','temp':'','volt':'','logging':'','session':'','filelist':'','position':''}        

        #self.setDebug()
        test['connection'] = self.getConnection()
        test['temp'] = self.getTemp()
        test['volt'] = self.getVolt()
        test['logging'] = self.getLogging()
        test['sessions'] = self.getSessions()
        test['filelist'] = self.getFileList()
        test['position'] = self.getPosition()

        return test

    
    def __getStuff(self, get_type, recv_dir='',name=''):
        """Method that basically is a big switch statement and is intended to minimize copy-paste in the logical code. 
            For command reference, consult http://www.septentrio.com/sup/secure/polarx3_2_2/secureDownload.php and 
            http://www.septentrio.com/sup/secure/polarx3_2_2/PolaRx2CmdRef.pdf"""        

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc

        # 1 # Define local variables
        result = {'value':None,'error':None}
        
        # 2 # Define command based on get_type
        if get_type =='sessions':
            url_string = 'http://%s/cgi-bin/anycmd?anycmd=gmi' % self.ipport
        if get_type =='position':
            url_string = 'http://%s/cgi-bin/anycmd?anycmd=grp' % self.ipport
        if get_type =='filelist':
            url_string = 'http://%s/cgi-bin/anycmd?anycmd=gfl' % self.ipport
        if get_type =='tracking':
            url_string = 'http://%s/cgi-bin/anycmd?anycmd=sdo' % self.ipport
        if get_type =='firmwareversion':
            url_string = 'http://%s/cgi-bin/anycmd?anycmd=gvf' % self.ipport

        

        if self.DEBUG: print 'URL string: %s' % url_string

        if self.CONNECTION_STATUS['router'] == True and self.CONNECTION_STATUS['receiver'] == True:
            if self.DEBUG: print "%s Both receiver (stat:%s) and router (stat:%s) are alive.. now some action!" % (currFunc,self.CONNECTION_STATUS['router'],self.CONNECTION_STATUS['receiver'])


            # 3 # Some CURL magic
            if url_string != '':
                if self.DEBUG: print "%s This is the URL string: %s " % (currFunc,url_string)        
                result = self.curl.getURL(url_string,self.userpass,self.DEBUG)
            else:
                print "No URL string defined..."

            # 4 # Some parsing magic
            if self.DEBUG: print '%s Before parsing: %s ' % (currFunc,result['value'])
            if result['value']:
                result['value'] = self.__parseFancy(get_type,result['value'])        
            if self.DEBUG: print '%s After parsing: %s ' % (currFunc,result['value'])         

        else:

            result['value'] = None

            if not self.CONNECTION_STATUS['router']:

                result['error'] = "Connection ERROR: ROUTER OFFLINE"
            else:
                result['error'] = "Connection ERROR: RECEIVER UNREACHABLE"


        # 5 # Return
        return result


    def __parseFancy(self, string_type, string):
        """Parses the return strings from the receiver and retuns only the relevant information in a dictionary """

        currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'
        if self.DEBUG: print "%s is now running..." % currFunc
        
        # 1 # Define variables
        result ='NOT REALLY...'

        if self.DEBUG: print '>>> String type: %s' % string_type

        # 2 # If string type is 'sessions'
        if string_type == 'sessions':        

            # 2.1 # Create a regexp comparison pattern 
            regex=re.compile(".(\d+.\d+).")

            # 2.2 # Try regexp comparison
            try:
                for line in string.split('\n'):
                    if self.DEBUG: print "LINE >>> ", line
                    if '$PolaRx:' in line:
                        if self.DEBUG: print 'Line found: ', line

                        # 2.3.1 # Run regexp
                        p = regex.search(line)         
                        if self.DEBUG: print "p.groups(): ", p.groups()                   

                        # 2.3.2 # Add resulting groups to dictionary
                        result = {'interval':p.group(1)}

                        if self.DEBUG: print '>>> SEARCHES: %s' % result


            except:
                if self.DEBUG: print 'ERROR: Pattern matching in %s failed..' % string_type 

        # 3 # If string type is 'filelist'
        if string_type == 'filelist':

            # 3.1 # Define variables
            result=[]
            patt_found = ''

            # 3.2 # Define regexp pattern 
            regex=re.compile('.(\w{3}).(\d{2}).(\d{4})\s(\d{2}):(\d{2})\s+(\d+)\s*.(PLRX\d{4}\.\d{2}_\.\w{3}).')

            # 3.3 # Define local variables 
            # Months dict is used to replace word-month for number-month                
            months = {  'Jan':'01',
                        'Feb':'02',
                        'Mar':'03',
                        'Apr':'04',
                        'May':'05',
                        'Jun':'06',
                        'Jul':'07',
                        'Aug':'08',
                        'Sep':'09',
                        'Oct':'10',
                        'Nov':'11',
                        'Dec':'12'}
            
            # List to collect dicts in..
            result = []    
                
            # 3.4 # Try regexp comparison
            try:
                for line in string.split('\n'):
                    if 'PolaR&' in line:
                        if self.DEBUG: print 'Line: ', line

                        # 3.4.1 # Run regexp
                        p = regex.search(line)                            

                        # 3.4.2 # Add resulting groups to dictionary
                        p_dict = {'month':months[p.group(1)],
                                    'day':p.group(2),
                                    'year':p.group(3),
                                    'hour':p.group(4),
                                    'minute':p.group(5),
                                    'size':p.group(6),
                                    'name':p.group(7)}

                        if self.DEBUG: print '>>> SEARCHES: %s' % p_dict

                        # 3.4.3 # Add each dict to the result list
                        result.append(p_dict)

            except:
                        
                if self.DEBUG: print 'ERROR: Pattern matching in %s failed..' % string_type 

        # 4 # If string type is 'position'
        if string_type == 'position':        

            # 4.1 # Define variables
            result={'latitude':None,'longitude':None,'altitude':None}
            patt_found = ''

            # 4.2 # Define regexp pattern
            regex=re.compile("(\d+.\d+),\s+(-\d+.\d+),\s+(\d+.\d+)")

            # 4.3 # Try regexp comparison
            try:
                for line in string.split('\n'):
                    if self.DEBUG: print "LINE >>> ", line
                    if '$PolaR#:' in line:
                        if self.DEBUG: print 'Line found: ', line

                        # 4.3.1 # Run regexp
                        p = regex.search(line)         
                        if self.DEBUG: print "p.groups(): ", p.groups()                   

                        # 4.3.2 # Add resulting groups to dictionary
                        result = {'latitude':p.group(1),'longitude':p.group(2),'altitude':p.group(3)}

                        if self.DEBUG: print '>>> SEARCHES: %s' % result


            except:
                if self.DEBUG: print 'ERROR: Pattern matching in %s failed..' % string_type 

        # 5 # If string type is 'tracking'
        if string_type == 'tracking':        

            # 5.1 # Define variables
            result=[]
            patt_found = ''

            # 5.2 # Define regexp pattern
            regex=re.compile("(\d+.\d+),\s+(-\d+.\d+),\s+(\d+.\d+)")

            # 5.3 # Try regexp comparison
            try:
                for line in string.split('\n'):
                    if self.DEBUG: print "LINE >>> ", line
                    #if '$PolaR#:' in line:
                     #   if self.DEBUG: print 'Line found: ', line

                      #  # 5.2.1 # Run regexp
                      #  p = regex.search(line)         
                      #  if self.DEBUG: print "p.groups(): ", p.groups()                   

                        # 5.2.2 # Add resulting groups to dictionary
                      #  result = {'latitude':p.group(1),'logitude':p.group(2),'altitude':p.group(3)}

                      #  if self.DEBUG: print '>>> SEARCHES: %s' % result


            except:
                if self.DEBUG: print 'ERROR: Pattern matching in %s failed..' % string_type 

        # 6 # If string type is 'firmwareversion'
        if string_type == 'firmwareversion':        
 
            # 6.1 # Define variables
            result=[]
            patt_found = ''

            # 6.2 # Define regexp pattern
            regex=re.compile("\s*(\S+):\s*(\S+)\s*")

            # 6.3 # Try regexp comparison
            try:    
                for line in string.split('\n'):
                    if 'firmware' in line:
                        if self.DEBUG: print 'Line: ', line
                        patt_found=regex.findall(line)
                        if self.DEBUG: print '>>> SEARCHES: %s' % patt_found
                        result.append(dict(patt_found))

            except:
                if self.DEBUG: print 'ERROR: %s failed' % string_type        

            
        # 7 # Finally return the result which could have come from step 2-6.
        return result


# HERE IS THE END!



from setuptools import setup, find_packages

setup(name='receivers',
    version='1.0',
    description='Module that provides single interface to multiple GPS receiver types',
    url='https://gitlab.com/gpskings/gpslibrary/tree/master/receivers/dist',
    author='Fjalar Sigurdarson',
    author_email='fjalar@vedur.is',
    license='Icelandic Met Office',
    packages= find_packages(),
    )



# ###############################
#
# cparse.py 
#
# Code made by fjalar@vedur.is 
# Iceland Met Office
# 2013
#
# ###############################


######## IMPORT LIBRARIES ########
 

class Parser:

    # ALL TEXT HERE IN THE DOC NEEDS TO BE REVISED LATER ON
    """A class designed to ....
    
    Public functions:
      getStationInfo([station_id])          - Get information for a single station. If no id is passed
                                            - information for all the stations in the config will be returned.
      getStationList                        - Get list of all stations.
      getCheckcommConfig()                  --
      getGetdataConfig(()                   --
      getPreprocessConfig()                 --
      getProcessConfig()                    --
      getPostprocessConfig()                --
      getStatus()                           --
      setDebug()                            --

    Private functions:
      __getConfigPaths()            --
      __clear_station_dicts()       --
      __parse_checkcomm_config()    --
      __parse_getdata_config        -- 
      __parse_preprocess_config     --
      __parse_process_config        --
      __parse_postprocess_config        --

    """

    def __init__(self):
        
        # 1 # Predefine few global variables for the class
        self.DEBUG                  = False
        self.CFG_PATH_GLOBAL        = '/etc/cparser/cparser.conf'
        self.CFG_PATH_STATIONS      = None
        self.CFG_PATH_CHECKCOMM     = None
        self.CFG_PATH_GETDATA       = None
        self.CFG_PATH_PREPROCESS    = None
        self.CFG_PATH_PROCESS       = None
        self.CFG_PATH_POSTPROCESS   = None

        # Variables for station config
        self.ALL_STATIONS = None

        # This function opens a config file and loads values to all the CFG_PATH_* variables
        self.__getConfigPaths()


    # ------------------ setDebug() ------------------ #

    def setDebug(self):
        """Debug ON/OFF switch."""
        if self.DEBUG:
            self.DEBUG = False
            print('Debug OFF')
        else:
            self.DEBUG = True
            print('Debug ON'    )

    # ------------------ setDebug() ------------------ #

    def getStatus(self):
        """Prints global variables"""
        print('Global variable status:')
        print('')
        print('DEBUG                    = ',self.DEBUG)
        print('CONFIG_PATH_GLOBAL       = ',self.CFG_PATH_GLOBAL)
        print('CONFIG_PATH_STATIONS     = ',self.CFG_PATH_STATIONS)
        print('CONFIG_PATH_CHECKCOMM    = ',self.CFG_PATH_CHECKCOMM)
        print('CONFIG_PATH_GETDATA      = ',self.CFG_PATH_GETDATA)
        print('CONFIG_PATH_PREPROCESS   = ',self.CFG_PATH_PREPROCESS)
        print('CONFIG_PATH_PROCESS      = ',self.CFG_PATH_PROCESS)
        print('CONFIG_PATH_POSTPROCESS  = ',self.CFG_PATH_POSTPROCESS)


    # ------------------ __getConfigPahts() ------------------ #

    def __getConfigPaths(self):
        # The parser config file should be located with the parser python file itself, in same directory.

        config_file = self.CFG_PATH_GLOBAL
        config = {'id':'','global_path':'','stations':'','checkcomm':'','getdata':'','preprocess':'','process':'','postprocess':''}

        try:
            f = open(config_file,'r')
            for line in f:
            
                # Check for commented lines
                if line.startswith('#') or len(line) == 1:
                    pass
                else:
                    words = line.split()

                    # Break if end of line
                    if words[0] == 'EOC':  
                        break 

                    # Else remove the '=' just leaving the variable name (words[0]) and value (words[1])
                    else:
                        words.remove('=')

                    # Parse config
                    if words[0] == 'Config_ID': config['id'] = words[1]
                    #if words[0] == 'Config_GLOBAL_PATH': self.CFG_PATH_GLOBAL = words[1]
                    if words[0] == 'configPath_STATIONS': self.CFG_PATH_STATIONS = words[1]
                    if words[0] == 'configPath_CHECKCOMM': self.CFG_PATH_CHECKCOMM = words[1]
                    if words[0] == 'configPath_GETDATA': self.CFG_PATH_GETDATA = words[1]
                    if words[0] == 'configPath_PREPROCESS': self.CFG_PATH_PREPROCESS = words[1]
                    if words[0] == 'configPath_PROCESS': self.CFG_PATH_PROCESS = words[1]
                    if words[0] == 'configPath_POSTPROCESS': self.CFG_PATH_POSTPROCESS = words[1]

            if config['id'] != 'parser':
                print('configParser.__getConfigPaths() > Incorrect file or file corrupted!')


        except IOError:
            print('')
            print('configParser.__getConfigPaths() > File (%s) missing!' % config_file)




    # ------------------ getStationConfig() ------------------ #

    def getStationInfo(self,station_id=''):

        stationList = [] # List of all info dictionaries, which contain few different other dictionaries.
        station, receiver, connection, proxy, router, serial, dialup, info = self.__clear_station_dicts() # get clean dictionaries
        stat_count = 0

        try:
            f = open(self.CFG_PATH_STATIONS,'r')

            for line in f:
                if line.startswith('#') or len(line) == 1:
                    #print('line passed')
                    pass
                else:
                    words = line.split()

                    if words[0] == 'EOF':  
                        pass 
                    else:
                        words.remove('=')

                    if words[0] == 'Station_NAME':
                        # Check if station id has been defined before.. which means this station name is new..                         
                        if station['name'] != '':

                            # Pack last station into the info dictionary..
                            info['station'] = station 
                            info['receiver'] = receiver
                            info['connection'] = connection
                            info['proxy'] = proxy
                            info['router'] = router
                            info['serial'] = serial
                            info['dialup'] = dialup

                            # Add the info dictionary to the station list
                            stationList.append(info)

                            if self.DEBUG: print('configParser.parse() >> Station %s appended..' % station['id'])

                            # Reset all dictionaries
                            station, receiver, connection, proxy, router, serial, dialup, info = self.__clear_station_dicts()        
                  

                        else:
                            station['name'] = words[0]
                            station['id']   = words[1]
                    # Station
                    if words[0] == 'Station_NAME': station['name'] = words[1]
                    if words[0] == 'Station_ID': station['id'] = words[1]
                    # Receiver
                    if words[0] == 'Receiver_TYPE': receiver['type'] = words[1]
                    if words[0] == 'Receiver_IP': receiver['ip'] = words[1]
                    if words[0] == 'Receiver_HTTP_PORT': receiver['httpport'] = words[1]            
                    if words[0] == 'Receiver_FTP_PORT': receiver['ftpport'] = words[1]            
                    if words[0] == 'Receiver_TELNET_PORT': receiver['telnetport'] = words[1]            
                    if words[0] == 'Receiver_SERIAL_PORT': receiver['serialport'] = words[1]            
                    if words[0] == 'Receiver_RTP_PORT': receiver['rtpport'] = words[1]            
                    if words[0] == 'Receiver_REFTEK_PORT': receiver['reftekport'] = words[1]            
                    if words[0] == 'Receiver_USER': receiver['user'] = words[1]
                    if words[0] == 'Receiver_PASS': receiver['pwd'] = words[1]
                    if words[0] == 'Receiver_SERVER': receiver['serv'] = words[1]
                    if words[0] == 'Receiver_BAUDRATE': receiver['baud'] = words[1]
                    if words[0] == 'Receiver_PARITY': receiver['parity'] = words[1]
                    if words[0] == 'Receiver_RTSCTS': receiver['rtscts'] = words[1]
                    # Connect
                    if words[0] == 'Connection_TYPE': connection['type'] = words[1].split(',')
                    if words[0] == 'Connection_PORT': connection['port'] = words[1]
                    if words[0] == 'Connection_PPP': connection['ppp'] = words[1]
                    if words[0] == 'Connection_SSH_PORT': connection['sshport'] = words[1]
                    # Proxy
                    if words[0] == 'Proxy_USER': proxy['user'] = words[1] 
                    if words[0] == 'Proxy_SSH_SERVER': proxy['sshserver'] = words[1]
                    if words[0] == 'Proxy_HOST': proxy['host'] = words[1]
                    if words[0] == 'Proxy_HTTP_PORT': proxy['httpport'] = words[1]
                    if words[0] == 'Proxy_FTP_PORT': proxy['ftpport'] = words[1]
                    if words[0] == 'Proxy_SERIAL_PORT': proxy['serialport'] = words[1]
                    # Router
                    if words[0] == 'Router_TYPE': router['type'] = words[1]
                    if words[0] == 'Router_IP': router['ip'] = words[1]
                    if words[0] == 'Router_PHONE_NUMBER': router['phonenumber'] = words[1]
                    if words[0] == 'Router_PORT_FORWARD': router['portforward'] = words[1]
                    # Serial
                    if words[0] == 'Serial_DEVICE': serial['device'] = words[1]
                    # Dialup
                    if words[0] == 'Dialup_DEVICE': dialup['device'] = words[1]
                    if words[0] == 'Dialup_PHONE_NUMBER': dialup['phonenumber'] = words[1]
                    if words[0] == 'Dialup_METH': dialup['meth'] = words[1]
                    if words[0] == 'Dialup_SERVER': dialup['server'] = words[1]
                    # End of file...
                    if words[0] == 'EOF':

                        # Wrap everything up
                        info['station']     = station 
                        info['receiver']    = receiver
                        info['connection']  = connection
                        info['proxy']       = proxy
                        info['router']      = router
                        info['serial']      = serial
                        info['dialup']      = dialup

                        # Append the station to the list
                        stationList.append(info)    

            info = {'station':'','receiver':'','connection':'','rotuer':'','dialup':''}

            if station_id == '':
                self.ALL_STATIONS = stationList
                #print(self.ALL_STATIONS)
                return stationList
            else:
                singleStationInfo = self.__get_single_station(station_id,stationList)
                return singleStationInfo


        except IOError:
            print('configParser.getStationInfo() > File (%s) missing!' % self.CFG_PATH_STATIONS)
        
    # ------------------ getStationList() ------------------ #

    def getStationList(self):

        if not self.ALL_STATIONS: all_info = self.getStationInfo()

        station_list = []

        for s in self.ALL_STATIONS:
            station_list.append(s['station']['id'])

        return station_list



    # ------------------ __get__single_station() ------------------ #
 
    def __get_single_station(self,station_id,stationList):
        for stationInfo in stationList:
            if stationInfo['station']['id'] == station_id:
                return stationInfo
        

    # ------------------ __clear_station_dicts() ------------------ #

    def __clear_station_dicts(self):

        station = { 'id':'',
                    'name':''}

        receiver= { 'type':'',
                    'ip':'',
                    'httpport':'',
                    'ftpport':'',
                    'telnetport':'',
                    'serialport':'',
                    'rtpport':'',
                    'reftekport':'',
                    'user':'',
                    'pwd':'',
                    'serv':'',
                    'baud':'',
                    'parity':'',
                    'rtscts':''}

        connection = {  'type':'',
                        'ppp':'',
                        'port':'',
                        'sshport':''}

        proxy = {   'user':'',
                    'sshserver':'',
                    'host':'',
                    'httpport':'',
                    'ftpport':'',
                    'serialport':''}

        router = {  'type':'',
                    'ip':'',
                    'phonenumber':'',
                    'portforward':''}

        serial = {  'device':''}

        dialup = {  'device':'',
                    'phonenumber':'',
                    'meth':'',
                    'server':''}

        info = {    'station':'',
                    'receiver':'',
                    'connect':'',
                    'proxy':'',
                    'router':'',
                    'serial':'',
                    'dialup':''} 

        return station, receiver, connection, proxy, router, serial, dialup, info


    # ------------------ getCheckcommConfig() ------------------ #

    def getCheckcommInfo(self):
        print('configParser.getCheckcommConfig > Not yet implemented!')

    # ------------------ getGetdataConfig() ------------------ #

    def getDataInfo(self,session_id=''):

        if self.DEBUG: print('configParser.getGetdataConfig() > Running')
        sessionList = [] # List of all info dictionaries, which contain few different other dictionaries.
        session = self.__clear_getdata_dicts() # get clean dictionaries
        stat_count = 0
        config = {'id':''}

        try:
            f = open(self.CFG_PATH_GETDATA,'r')
            if self.DEBUG: print('configParser.getGetdataConfig() > Config file available and open...')

            for line in f:
                if self.DEBUG: print('configParser.getGetdataConfig() > Traversing lines in file...')
                if line.startswith('#') or len(line) == 1:
                    #print('line passed')
                    if self.DEBUG: print('configParser.getGetdataConfig() > Passing line: ',line)
                    #pass
                else:
                    if self.DEBUG: print('configParser.getGetdataConfig() > Splitting line into words...')
                    words = line.split()

                    if words[0] == 'EOF':
                        if self.DEBUG: print('configParser.getGetdataConfig() > EOF found...exiting.'  )
                        pass 
                    else:
                        if self.DEBUG: print('configParser.getGetdataConfig() > Stripping "=" away..')
                        words.remove('=')

                    if words[0] == 'Session_ID':
                        if self.DEBUG: print('configParser.getGetdataConfig() > Session ID >%s< found..' % words[1])

                        # Check if session id has been defined before.. which means this session id is new..                         
                        if session['id'] != '':

                            # Add the session dictionary to the station list
                            sessionList.append(session)

                            if self.DEBUG: print('configParser.getGetdataConfig() > Session %s appended to sessionList' % session['id'])

                            # Reset all dictionaries
                            session = self.__clear_getdata_dicts()  
                            if self.DEBUG: print('configParser.getGetdataConfig() > Session list cleared for the next session...'  )

                            # Save the next session ID to the new session
                            if self.DEBUG: print('configParser.getGetdataConfig() > Next session created.' )
                            session['id']   = words[1]    
                  

                        else:
                            session['id']   = words[1]

                    # Session
                    if self.DEBUG: print('configParser.getGetdataConfig() > Parsing list...')

                    if words[0] == 'Config_ID': config['id'] = words[1]
                    if words[0] == 'Session_ID': session['id'] = words[1]
                    if words[0] == 'Destination_PATH': session['destpath'] = words[1]
                    if words[0] == 'Sample_FREQUENCY': session['samplfreq'] = words[1]
                    if words[0] == 'Store_To_DB': session['storetodb'] = words[1]
                    if words[0] == 'Report_LEVEL': session['reportlevel'] = words[1]

                    if words[0] == 'Days_To_DOWNLOAD': session['daystod'] = words[1]


                    # End of file...
                    if words[0] == 'EOF':

                        # Append the station to the list
                        sessionList.append(session)    

            if config['id'] != 'getdata':

                print('configParser.getGetdataConfig() > Incorrect file or file corrupted!')
                print('configParser.getGetdataConfig() > Config ID: ',config['id'] )
                print('configParser.getGetdataConfig() > sessionList[0]: ',sessionList[0])

            else:

                if session_id == '':
                    return sessionList
                else:
                    singleSessionInfo = self.__get_single_session(session_id,sessionList)
                    return singleSessionInfo

        except IOError:
            print('configParser.getGetdataConfig() > File (%s) missing!' % self.CFG_PATH_GETDATA)
            

    # ------------------ __get__single_station() ------------------ #
 
    def __get_single_session(self,session_id,sessionList):
        for sessionInfo in sessionList:
            if sessionInfo['id'] == session_id:
                return sessionInfo



    # ------------------ __clear_getdata_dicts() ------------------ #

    def __clear_getdata_dicts(self):

        session = { 'id':'',
                    'destpath':'',
                    'samplfreq':'',
                    'storetodb':'',
                    'reportlevel':'',
                    'daystod':''}

        return session
       


    # ------------------ getPreprocessConfig() ------------------ #

    def getPreprocessConfig(self):
        print('configParser.getCheckcommConfig > Not yet implemented!')

    # ------------------ getProcessConfig() ------------------ #

    def getProcessConfig(self):
        print('configParser.getCheckcommConfig > Not yet implemented!')

    # ------------------ getPostprocessConfig() ------------------ #

    def getPostprocessConfig(self):
        """
        Retreive config informaton for gamit time series manupulations
        returns config dictionary.
        currently implementing 
        {'figDir': standard location for saving figures,
         'prePath': path of older time series data,
         'rapPath': path of time series data from last globk run,
         'totPath': path of the compined time series prePath + rapPath,
         'fTypes': list of figure types to be saved currently png ad ppm
         'pfile': Name of a file containing station plate information
         'coordfile': Name of a file containing station coordinates
         'tiefile': Name of a file containing station ties
         'detrendfile': Name of a file containing detrend information
         }

        how time series are handled needs to reveised prePath, rapPath, totPath config will therefore change soon.

        Implemented by bgo@vedur.is, Nov. 2013

        """

        import os

        config_file = self.CFG_PATH_POSTPROCESS
        config = {'figDir':'','prePath':'','rapPath':'','totPath':'','fTypes':'','pfile':'','coordfile':'','tiefile':'','detrendfile':''}

        try:
            f = open(config_file,'r')
            for line in f:

                # Check for commented lines
                if line.startswith('#') or len(line) == 1:
                    pass
                else:
                    words = line.split()

                    # Break if end of line
                    if words[0] == 'EOC':  
                        break 

                    # Else remove the '=' just leaving the variable name (words[0]) and value (words[1])
                    else:
                        words.remove('=')

                    # Parse config
                    if words[0] == 'figDir': config['figDir'] = os.path.expanduser(words[1])
                    if words[0] == 'prePath': config['prePath'] = os.path.expanduser(words[1])
                    if words[0] == 'rapPath': config['rapPath'] = os.path.expanduser(words[1])
                    if words[0] == 'totPath': config['totPath'] = os.path.expanduser(words[1])
                    if words[0] == 'fTypes': config['fTypes'] = os.path.expanduser(words[1])
                    if words[0] == 'pfile': config['pfile'] = os.path.expanduser(words[1])
                    if words[0] == 'coordfile': config['coordfile'] = os.path.expanduser(words[1])
                    if words[0] == 'tiefile': config['tiefile'] = os.path.expanduser(words[1])
                    if words[0] == 'detremdfile': config['detrenddfile'] = os.path.expanduser(words[1])

        except IOError:
            print('configParser.getGetdataConfig() > File (%s) missing!' % config_file)

        return config


    # The kill function for the object
    def kill(self):
        del self





from setuptools import setup, find_packages
with open("README.md", "r") as fh:
    long_description = fh.read()
setup(name='geo_dataread',
    version='0.1.0',
    author='Benedikt G. Ofeigsson',
    author_email='bgo@vedur.is',
    description='Reads GPS, multigas, seismic, hydrological datasets',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://gitlab.com/gpskings/gpslibrary.git',
    license='Icelandic Met Office',
    package_dir={'geo_dataread': 'geo_dataread'},
    scripts=['bin/read_sil_data'],
    packages=find_packages(),
    zip_safe=False,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License :: Icelandic Met Office",
        "Operating System :: POSIX",
    ],
)



"""
This module contains functions for reading and returning GPS data

The module containes the following functions 
---------------
openGlobkTimes(STA, Dir=None)
dPeriod(yearf, data, Ddata, startyear=None, endyear=None)
iprep(yearf, data, Ddata, uncert=20.0)
detrend(yearf, data, fitfunc, errfunc, STA)
vshift(yearf, data, Ddata, uncert=20.0, refdate=None, Period=5):
filt_outl(yearf,data,Ddata,pb,errfunc,outlier)
gamittoNEU(sta):
---------------

"""

import numpy as np
import logging

#
# time series filtering
#


def line(x, p0, p1):
    return p0 + p1 * x


def lineperiodic(x, p0, p1, p2, p3, p4, p5):
    """
    linear function with a periodic supperimposed
    """

    import numpy as np

    return (
        p0
        + p1 * x
        + p2 * np.cos(2 * np.pi * x)
        + p3 * np.sin(2 * np.pi * x)
        + p4 * np.cos(4 * np.pi * x)
        + p5 * np.sin(4 * np.pi * x)
    )


def periodic(x, p0, p1, p2, p3, p4, p5):
    """
    Periodic function
    """

    import numpy as np

    return (
        p2 * np.cos(2 * np.pi * x)
        + p3 * np.sin(2 * np.pi * x)
        + p4 * np.cos(4 * np.pi * x)
        + p5 * np.sin(4 * np.pi * x)
    )


def xf(x, p0, p1, p2, tau=4.8):
    import numpy as np

    return p0 + p1 * x + p2 * np.exp(-tau * x)


def expxf(x, p0, p1, p2, p3):
    import numpy as np

    return p0 + p1 * x + p2 * np.exp(-p3 * x)


def expf(x, p0, p1, p2):
    import numpy as np

    return p0 + p1 * np.exp(-p2 * x)


def gpsvelo_df():
    """
    dataframe for pygmt velo with three  extra columns
    for date, period and bolian for vertical data
    """
    import pandas as pd

    gpsvelo = pd.DataFrame(
        columns=[
            "longitude",
            "latitude",
            "east_velo",
            "north_velo",
            "east_sigma",
            "north_sigma",
            "coorelation_EN",
            "Station",
            "date",
            "period",
            "vertical",
        ],
    )

    return gpsvelo


def gpsvelo(sta, ll, vel, vertical=False, vfile=None, pheader=False):
    """
    Return gps velocities gmt velo
    """

    gpsvelo = "{0:5.6f} {1:5.6f}\t{2:7.2f} {5:7.2f}\t{3:7.2f} {6:7.2f}\t{4:7.2f} {7:7.2f}\t\t{8:s}".format(
        ll[1], ll[0], vel[0], vel[1], vel[2], vel[3], vel[4], vel[5], sta
    )


def getDetrFit(
    STA,
    useSTA=None,
    useFIT=None,
    onlyPeriodic=False,
    detrFile="detrend_itrf2008.csv",
    logging_level=logging.WARNING,
):
    """ """

    import logging
    import numpy as np
    import pandas as pd

    import cparser as cp

    # Handling logging
    logging.basicConfig(
        format="%(asctime)-15s [%(levelname)s] %(funcName)s: %(message)s",
        level=logging_level,
    )

    logging.getLogger().setLevel(logging_level)
    module_logger = logging.getLogger()

    North = ["Nrate", "Nacos", "Nasin", "Nscos", "Nssin"]
    East = ["Erate", "Eacos", "Easin", "Escos", "Essin"]
    Up = ["Urate", "Uacos", "Uasin", "Uscos", "Ussin"]
    Info = ["Sitename", "Starttime", "Endtime", "UseSTA", "Fit"]
    columns = North + East + Up + Info
    sta_name = cp.Parser().getStationInfo(STA)["station"]["name"]
    data = [[np.nan] * 15 + [sta_name, np.nan, np.nan, np.nan, np.nan]]
    table = pd.DataFrame(data, index=[STA], columns=columns)

    try:
        table = pd.read_csv(detrFile, index_col="STA")
    except pd.errors.EmptyDataError:
        module_logger.warn("{} does not contain any data".format(detrFile))
    except FileNotFoundError:
        module_logger.warn("Detrend file not found")

    module_logger.info("Existing detrend constant table:\n{}".format(table))
    try:
        const = pd.DataFrame(index=[STA], columns=columns)
        const.loc[STA] = tuple(table.loc[STA].values)
    except KeyError:
        module_logger.warn("{} not in table making an empty row".format(STA))
        const = pd.DataFrame(data, index=[STA], columns=table.columns)

    if useSTA:
        module_logger.warn(
            "Setting {0} {1} constants as {1} parameters for {2}".format(
                useSTA, useFIT, STA
            )
        )
        module_logger.info("current {} constants are:\n {} ".format(STA, const))

        const.loc[STA, ["UseSTA", "Fit"]] = useSTA, useFIT
        if useFIT == "periodic":
            const.loc[STA, North[1:] + East[1:] + Up[1:]] = table.loc[
                useSTA, North[1:] + East[1:] + Up[1:]
            ]
        elif useFIT == "lineperiodic":
            const.loc[STA, North + East + Up] = table.loc[
                useSTA, North + East + Up
            ]
        elif useFIT == "line":
            const.loc[STA, North[:1] + East[:1] + Up[:1]] = table.loc[
                useSTA, North[:1] + East[:1] + Up[:1]
            ]

        module_logger.info("New parameters for {} are:\n {}".format(STA, const))

    if onlyPeriodic is True:
        const["Nrate"] = const["Erate"] = const["Urate"] = 0

    return const


def convconst(const, pb=None):
    """
    convert detrend constants from pandas dataframe to list of the form
    [[], [], []] for optimize.curv_fit
    """

    import numpy as np

    North = ["Nrate", "Nacos", "Nasin", "Nscos", "Nssin"]
    East = ["Erate", "Eacos", "Easin", "Escos", "Essin"]
    Up = ["Urate", "Uacos", "Uasin", "Uscos", "Ussin"]
    if pb:
        # const = pd.DataFrame(data, index=[STA], columns=table.columns)
        if len(pb[0]) == 2:
            pb = [np.concatenate((pb[i], np.zeros(4))) for i in range(3)]

        stat = const.index.tolist()[0]
        for i, dimention in zip(range(3), [North, East, Up]):
            const.loc[stat, dimention] = list(pb[i][1:])

        return const

    else:
        if const[North].isna().sum(axis=1).values == 5:
            p0 = [None, None, None]
        else:
            p0 = [[], [], []]
            p0[0] = [0] + const[North].values.tolist()[0]
            p0[1] = [0] + const[East].values.tolist()[0]
            p0[2] = [0] + const[Up].values.tolist()[0]

            p0 = np.nan_to_num(p0)

        return p0


def save_detrend_const(const, detrFile="detrend_itrf2008.csv"):
    """
    save a constants for detrending GPS stations
    """
    import pandas as pd
    from pathlib import Path

    North = ["Nrate", "Nacos", "Nasin", "Nscos", "Nssin"]
    East = ["Erate", "Eacos", "Easin", "Escos", "Essin"]
    Up = ["Urate", "Uacos", "Uasin", "Uscos", "Ussin"]
    Info = ["Sitename", "Starttime", "Endtime", "UseSTA", "Fit"]
    columns = North + East + Up + Info

    stationlist = const.index.values
    path = Path(Path.cwd(), detrFile)
    if not path.is_file():
        open(path, "w").close()

    try:
        table = pd.read_csv(detrFile, index_col="STA")
    except pd.errors.EmptyDataError:
        table = pd.DataFrame(index=["STA"], columns=columns)

    if all(table.isnull().all(1)):
        print("No data in file")
        const.to_csv(detrFile, mode="w", header=True, index_label="STA")

        return
    else:
        for stat in stationlist:
            if stat in table.index:
                print("Station {} already in table updating".format(stat))
                table.loc[stat, :] = const.loc[stat]
                print("Saving the new table")
                table.to_csv(detrFile, mode="w", header=True, index_label="STA")
            else:
                print("stations {} are not in detrend file, adding it".format(stat))
                table.loc[stat, :] = const.loc[stat]
                table.to_csv(detrFile, mode="w", header=True, index_label="STA")

        return


def fitdataframe(func, df, p0=[None, None, None]):
    """ """
    from scipy import optimize

    x = df["yearf"].to_numpy()
    y = df[["north", "east", "up"]].to_numpy().T
    yD = df[["Dnorth", "Deast", "Dup"]].to_numpy().T

    return fittimes(func, x, y, yD, p0=p0)


def fittimes(func, x, y, yD=[None, None, None], p0=[None, None, None]):
    from scipy import optimize

    pb = [[], [], []]
    pcov = [[], [], []]

    for i in range(3):
        pb[i], pcov[i] = optimize.curve_fit(
            func, x, y[i], p0=p0[i], sigma=yD[i], maxfev=100000
        )

    return pb, pcov


# routines to extract and save coordiantes and time series from gamit


def gamittooneuf(sta, outFile, mm=True, ref="plate", dstring=None, outformat=True):
    """
    extract Gamit time series from standard format to one file formated time string
    input:
        sta: station four letter short name
        outFile: file object
        mm: boolean True return data in mm, else in m
        ref: subtract plate velocity from the time series.
        dstring: format of the time string (e.g dstring)="%Y%m%d-%H%M%S"), defaults to decimal year (yyyy.yyyy)
        outformat: not finished but determains the output order of the data into the file
    """
    # "%Y/%m/%d 12:00:00.000"

    NEUdata = gamittoNEU(sta, mm=mm, ref=ref, dstring=dstring)

    gamittoFile(NEUdata, outFile, mm=mm, ref=ref, dstring=dstring, outformat=outformat)


def gamittoFile(NEUdata, outFile, mm=True, ref="plate", dstring=None, outformat=True):
    """ """

    if dstring == "yearf":
        timef = "{0: 8.4f}\t"
        timeh = "#\"yyyy.dddd'     "
    else:
        timef = " {0:s}\t"
        timeh = "#yyyy/mm/dd HH:MM:SS.SSS          "

    if mm:
        if outformat:
            header = timeh + "dN[mm] DN[mm]\tdE[mm] DE[mm]\tdU[mm]  DU[mm]"
            formatstr = (
                timef + "{1: 7.2f} {4: 7.2f}\t{2: 7.2f} {5: 7.2f}\t{3: 7.2f} {6: 7.2f}"
            )
        else:
            header = timeh + "dN[mm]  dE[mm] dU[mm]\t\t  DN[mm]  DE[mm]  DU[mm]"
            formatstr = (
                timef
                + "{1: 7.2f} {2: 7.2f} {3: 7.2f}\t\t{4: 7.2f} {5: 7.2f}\t{6: 7.2f}"
            )
    else:
        if outformat:
            header = timeh + "dN[m] DN[m]\tdE[m] DE[m]\tdU[m]  DU[m]"
            formatstr = (
                timef + "{1: 7.5f} {4: 7.5f}\t{2: 7.5f} {5: 7.5f}\t{3: 7.5f} {6: 7.5f}"
            )
        else:
            header = timeh + "dN[m]    dE[m]    dU[m]          DN[m]   DE[m]    DU[m]"
            formatstr = (
                timef + "{1: 7.5f} {2: 7.5f} {3: 7.5f}\t{4: 7.5f} {5: 7.5f} {6: 7.5f}"
            )

    print(header, file=outFile)  # ,

    for x in NEUdata:
        print(
            formatstr.format(
                x["yearf"],
                x["data[0]"],
                x["data[1]"],
                x["data[2]"],
                x["Ddata[0]"],
                x["Ddata[1]"],
                x["Ddata[2]"],
            ),
            file=outFile,
        )


def savedisp(dataDict, fname=None, header=""):
    """ """
    from collections import OrderedDict

    valtype = type(list(dataDict.values())[0])

    dataDict = OrderedDict(sorted(dataDict.items()))

    datavalues = list(dataDict.values())
    if (valtype is list) or (valtype is np.ndarray):
        fmt = "% 3.8f\t% 2.8f\t% 2.8f\t%s"
        ab = np.zeros(
            len(dataDict.keys()),
            dtype=[
                ("var1", "float"),
                ("var2", "float"),
                ("var3", "float"),
                ("var4", "a4"),
            ],
        )
        ab["var1"] = np.squeeze(datavalues)[:, 0]
        ab["var2"] = np.squeeze(datavalues)[:, 1]
        ab["var3"] = np.squeeze(datavalues)[:, 2]
        ab["var4"] = [str(item) for item in dataDict.keys()]
    if valtype is tuple:
        fmt = "% 3.8f\t% 2.8f\t% 2.8f\t%2.8f\t%2.8f\t%s"
        ab = np.zeros(
            len(dataDict.keys()),
            dtype=[
                ("var1", "float"),
                ("var2", "float"),
                ("var3", "float"),
                ("var4", "float"),
                ("var5", "float"),
                ("var6", "a4"),
            ],
        )
        ab["var1"] = np.squeeze(list(zip(*datavalues[:]))[0])[:, 0]
        ab["var2"] = np.squeeze(list(zip(*datavalues[:]))[0])[:, 1]
        ab["var3"] = np.squeeze(list(zip(*datavalues[:]))[1])[:, 0]
        ab["var4"] = np.squeeze(list(zip(*datavalues[:]))[1])[:, 1]
        ab["var5"] = np.squeeze(list(zip(*datavalues[:]))[1])[:, 2]
        ab["var6"] = [str(item) for item in dataDict.keys()]

    if fname:
        np.savetxt(fname, ab, fmt=fmt, header=header)
    return ab


def extractfromGamitBakf(cfile, stations):
    """ """
    import re

    slines = []

    site = re.compile(stations)
    tim = re.compile("Solution refers to", re.IGNORECASE)
    f = open(cfile, "r")

    for line in f:
        if site.search(line):  # or tim.search(line):
            slines.append(line.rstrip())

    return slines


def openGlobkTimes(STA, Dir=None, tType="TOT"):
    """
    Import data from Globk time series files into a numpy arrays

    Dir is the directory containing the time series if left blank the default path will will be the path
    defined in config file postprossesing.cfg, totPath

    input:
        STA, Station four letter short name in captial letters
        Dir, optional alternative location of the GAMIT time series data.

    output:
         yearf, data, Ddata
            yearf: is array with time  (usually) in fractional year format (i.e. 2014.62328)
            data:  three arrays containing GPS data in north,east, up
            Ddata: respective uncertainty values

    """

    import os
    import numpy as np
    import datetime as dt

    # custom modules
    import cparser as cp
    from gtimes.timefunc import shifTime, TimetoYearf

    # loading the data-------------------

    if Dir is None:
        Dir = cp.Parser().getPostprocessConfig()["totPath"]
    else:
        # print( os.path.isdir(Dir) )
        # check if the path given as argument indeed exists
        pass

    # constructing the full path filenames and parsing parameters

    # filepre = "mb_%s_TOT.dat" % STA

    filepre = "mb_{0:s}_{1:s}.dat".format(STA, tType)
    if os.path.isfile(os.path.join(Dir, filepre + "1")):
        pass
    else:
        filepre = "mb_{0:s}_{1:s}.dat".format(STA, "TOT")

    Datafile1 = os.path.join(Dir, filepre + "1")
    Datafile2 = os.path.join(Dir, filepre + "2")
    Datafile3 = os.path.join(Dir, filepre + "3")

    # loading the data for files and storing it in arrays
    yearf, d1, D1 = np.loadtxt(
        Datafile1, unpack=True, skiprows=3, converters={1: __converter, 2: __converter}
    )
    d2, D2 = np.loadtxt(
        Datafile2,
        usecols=(1, 2),
        unpack=True,
        skiprows=3,
        converters={1: __converter, 2: __converter},
    )
    d3, D3 = np.loadtxt(
        Datafile3,
        usecols=(1, 2),
        unpack=True,
        skiprows=3,
        converters={1: __converter, 2: __converter},
    )
    data = np.vstack([d1, d2, d3])
    Ddata = np.vstack([D1, D2, D3])

    if tType == "08h":
        shift8h = dt.timedelta(**shifTime("H8"))
        yearf = np.array(
            [
                TimetoYearf(*(item + shift8h).timetuple()[:6])
                for item in toDateTime(yearf)
            ]
        )

    return yearf, data, Ddata


def open3DataFiles(STA, Dir=None, comp=["-N", "-E", "-U"]):
    """
    open data contained in 3 files one file for each component E, N and U
    """

    import os
    import pandas as pd

    if Dir is None:
        Dir = os.getcwd()

    compdict = {}
    compdict[comp[0]] = ["north", "Dnorth"]
    compdict[comp[1]] = ["east", "Deast"]
    compdict[comp[2]] = ["up", "Dup"]
    components = {"north": None, "east": None, "up": None}

    for item in compdict.keys():
        dfile = "{0}{1}".format(STA, item)
        components[compdict[item][0]] = pd.read_csv(
            dfile, sep=r'\s+', index_col=0, header=None, names=compdict[item]
        )

    # return reduce(lambda x, y: pd.merge(x, y, on=inpd.concat([df1, df2, df3, ...], axis=1)dex_col), components.values() )
    columnreorder = ["north", "east", "up", "Dnorth", "Deast", "Dup"]
    data = pd.concat(components.values(), axis=1)[columnreorder]
    data.set_index(pd.DatetimeIndex(toDateTime(data.index)), inplace=True)
    data.index = data.index.round("1h")

    return data


def convGlobktopandas(yearf, data, Ddata):
    """
    input: yearf, data, Data from openGlobkTimes
    output: pandas dataframe


    reads in time series data from openGlobkTimes and returns a dataframe with the structure
    Index [datetime], north, east, up, Dnorth, Deast, Dup, yearf
    """

    import pandas as pd
    from collections import OrderedDict

    # reduce(lambda x, y: pd.merge(x, y, on = 'Date'), dfList)
    names = ["north", "east", "up", "Dnorth", "Deast", "Dup", "yearf"]

    # !!!!! from_item depricated will remove
    # data = pd.DataFrame.from_items(zip(names[:3],data))
    # data = data.join( pd.DataFrame.from_items(zip(names[3:],Ddata) ) )
    # Using from_dict instead
    data = pd.DataFrame.from_dict(OrderedDict(zip(names[:3], data)))
    data = data.join(pd.DataFrame.from_dict(OrderedDict(zip(names[3:], Ddata))))
    data["yearf"] = yearf
    data.set_index(pd.DatetimeIndex(toDateTime(yearf)), inplace=True)
    data.index = data.index.round("1h")

    return data


def compGlobkTimes(stalist="any", dirConFilePath=None, freq=None):
    """
    joins old and new mb_ time series files
    """
    import glob
    import sys
    import shutil
    import os

    import cparser as cp

    if dirConFilePath:  # for custom file
        Dirs = parsedir(dirConFilePath)
    else:
        Dirs = cp.Parser().getPostprocessConfig()
        print(Dirs)

    PrePath = Dirs["prePath"]
    RapPath = Dirs["rapPath"]
    TotPath = Dirs["totPath"]

    if freq == "TOT" or freq is None:
        freq = "TOT"
    else:
        PrePath = PrePath + "_%s" % (freq)
        RapPath = RapPath + "_%s" % (freq)

    if stalist == "any":
        FilePreL = os.path.join(PrePath, "mb_*.dat?")
        FileRapL = os.path.join(RapPath, "mb_*.dat?")

        List = glob.glob(FilePreL) + glob.glob(FileRapL)

        # listing all stations in  the Rap and Pre dir
        stalist = sorted(set([item[-13:-9] for item in List]))

    for STA in stalist:
        FilePre = "mb_%s_?PS.dat" % STA
        OutFilePre = "mb_%s_GPS.dat" % STA
        GPS20PS = "mb_%s_0PS.dat" % STA

        for axes in range(1, 4):
            FilePreR = os.path.join(PrePath, FilePre + "%s" % (axes,))
            FileRapR = os.path.join(RapPath, FilePre + "%s" % (axes,))

            # graping the list for files for for that station
            PreFileL = glob.glob(FilePreR)  # listing files in the pre dir
            RapFileL = glob.glob(FileRapR)  # listing files in th Rap dir

            #  Sorting the file lists
            PreFileL.sort()
            if len(PreFileL) > 1:
                PreFileL.insert(0, PreFileL.pop(-1))
            RapFileL.sort()
            if len(RapFileL) > 1:
                RapFileL.insert(0, RapFileL.pop(-1))

            TotFile = os.path.join(TotPath, "mb_%s_%s.dat%s" % (STA, freq, axes))
            print("Concating all the %s data to %s" % (STA, TotFile))
            if os.path.exists(TotFile):
                os.remove(TotFile)
            outf = open(TotFile, "a")
            for fil in PreFileL:
                print("Processing file %s " % fil, file=sys.stderr)
                f = open(fil)
                f.seek(61)
                shutil.copyfileobj(f, outf)
                f.close()
            outf.close()

            preexist = os.stat(TotFile).st_size != 0
            if preexist:
                outf = open(TotFile, "r")
                lastline = outf.readlines()[-1]
                lastline = lastline.split()
                outf.close()

            outf = open(TotFile, "a")
            for file in RapFileL:
                formatstr = "Processing file {0:s} ".format(file)
                print(formatstr, file=sys.stderr)
                rapfile = open(file, "r")
                rapfile.seek(61)
                lines = rapfile.readlines()
                if preexist:
                    lines = "".join(
                        [line for line in lines if line.split()[0] > lastline[0]]
                    )
                else:
                    lines = "".join([line for line in lines])

            outf.close()


def TieTimes(sta1, sta2, dirConFilePath=None, freq=None, tie=[None, None, None]):
    """
    joins old and new mb_ time series files
    """
    import glob
    import sys
    import shutil

    import cparser as cp

    if dirConFilePath:  # for custom file
        Dirs = parsedir(dirConFilePath)
    else:
        Dirs = cp.Parser().getPostprocessConfig()

    PrePath = Dirs["prePath"]
    RapPath = Dirs["rapPath"]
    TotPath = Dirs["totPath"]

    if freq == "TOT" or freq is None:
        freq = "TOT"
    else:
        PrePath = PrePath + "_%s" % (freq)
        RapPath = RapPath + "_%s" % (freq)

    if stalist == "any":
        FilePreL = os.path.join(PrePath, "mb_*.dat?")
        FileRapL = os.path.join(RapPath, "mb_*.dat?")

        List = glob.glob(FilePreL) + glob.glob(FileRapL)

        # listing all stations in  the Rap and Pre dir
        stalist = sorted(set([item[-13:-9] for item in List]))

    for STA in stalist:
        FilePre = "mb_%s_?PS.dat" % STA
        OutFilePre = "mb_%s_GPS.dat" % STA
        GPS20PS = "mb_%s_0PS.dat" % STA

        for axes in range(1, 4):
            FilePreR = os.path.join(PrePath, FilePre + "%s" % (axes,))
            FileRapR = os.path.join(RapPath, FilePre + "%s" % (axes,))

            # graping the list for files for for that station
            PreFileL = glob.glob(FilePreR)  # listing files in the pre dir
            RapFileL = glob.glob(FileRapR)  # listing files in th Rap dir

            #  Sorting the file lists
            PreFileL.sort()
            if len(PreFileL) > 1:
                PreFileL.insert(0, PreFileL.pop(-1))
            RapFileL.sort()
            if len(RapFileL) > 1:
                RapFileL.insert(0, RapFileL.pop(-1))

            TotFile = os.path.join(TotPath, "mb_%s_%s.dat%s" % (STA, freq, axes))
            print("Concating all the %s data to %s" % (STA, TotFile))
            if os.path.exists(TotFile):
                os.remove(TotFile)
            outf = open(TotFile, "a")
            for fil in PreFileL:
                print("Processing file %s " % fil, file=sys.stderr)
                f = open(fil)
                f.seek(61)
                shutil.copyfileobj(f, outf)
                f.close()
            outf.close()

            preexist = os.stat(TotFile).st_size != 0
            if preexist:
                outf = open(TotFile, "r")
                lastline = outf.readlines()[-1]
                lastline = lastline.split()
                outf.close()

            outf = open(TotFile, "a")
            for file in RapFileL:
                formatstr = "Processing file {0:s} ".format(file)
                print(formatstr, file=sys.stderr)
                rapfile = open(file, "r")
                rapfile.seek(61)
                lines = rapfile.readlines()
                if preexist:
                    lines = "".join(
                        [line for line in lines if line.split()[0] > lastline[0]]
                    )
                else:
                    lines = "".join([line for line in lines])

                outf.write(lines)
                rapfile.close()

            outf.close()


def TieTimes(sta1, sta2, dirConFilePath=None, freq=None, tie=[None, None, None]):
    """
    joins old and new mb_ time series files
    """
    import glob
    import sys
    import shutil

    from pandas import read_table
    import pandas as pd

    import cparser as cp

    if dirConFilePath:  # for custom file
        Dirs = parsedir(dirConFilePath)
    else:
        Dirs = cp.Parser().getPostprocessConfig()

    # PrePath = Dirs['prePath']
    # RapPath = Dirs['rapPath']
    TieFile = Dirs["tiefile"]
    TotPath = Dirs["totPath"]

    if freq == "TOT" or freq is None:
        freq = "TOT"
    else:
        PrePath = PrePath + "_%s" % (freq)
        RapPath = RapPath + "_%s" % (freq)

    print(TieFile)

    dtype = [
        ("North", "<f8"),
        ("East", "<f8"),
        ("Up", "<f8"),
        ("sta1", "|S5"),
        ("sta2", "|S5"),
    ]

    const = np.genfromtxt(TieFile, dtype=dtype)
    const = [i for i in const if i[3] == sta1 and i[4] == sta2]

    for axes in range(1, 4):
        TotFile1 = os.path.join(TotPath, "mb_%s_%s.dat%s" % (sta1, freq, axes))
        TotFile2 = os.path.join(TotPath, "mb_%s_%s.dat%s" % (sta2, freq, axes))
        print("Concating all the %s data to %s" % (sta1, TotFile2))
        # outf = open(TotFile, 'r')
        data1 = read_table(
            TotFile1, sep=r'\s+', header=None, index_col=0, names=["disp", "uncert"]
        )
        data2 = pd.read_csv(
            TotFile2, sep=r'\s+', header=None, index_col=0, names=["disp", "uncert"]
        )
        print(const[0][axes - 1])
        data2["disp"] -= const[0][axes - 1] / 1000
        data = pd.concat([data1, data2])

        outfile = os.path.join(TotPath, "mb_%s_%s.dat%s" % (sta2, "JON", axes))
        data.to_csv(outfile, sep="\t", index=True, header=False)


fitfuncl = lambda p, x: p[0] * x + p[1]
errfuncl = lambda p, x, y: fitfuncl(p, x) - y  # Distance to the target function
fitfunc = (
    lambda p, x: p[0] * x
    + p[1] * np.cos(2 * np.pi * x)
    + p[2] * np.sin(2 * np.pi * x)
    + p[3] * np.cos(4 * np.pi * x)
    + p[4] * np.sin(4 * np.pi * x)
    + p[5]
)
errfunc = lambda p, x, y: fitfunc(p, x) - y  # Distance to the target function


def fitline(yearf, data, STA):
    """
    fit a line through data
    """

    from scipy import optimize
    import numpy as np

    dtype = [
        ("Nrate", "<f8"),
        ("Erate", "<f8"),
        ("Urate", "<f8"),
        ("Nacos", "<f8"),
        ("Nasin", "<f8"),
        ("Eacos", "<f8"),
        ("Easin", "<f8"),
        ("Uacos", "<f8"),
        ("Uasin", "<f8"),
        ("Nscos", "<f8"),
        ("Nssin", "<f8"),
        ("Escos", "<f8"),
        ("Essin", "<f8"),
        ("Uscos", "<f8"),
        ("Ussin", "<f8"),
        ("shortname", "|S5"),
        ("name", "|S20"),
    ]

    const = np.genfromtxt("itrf08det", dtype=dtype)
    const = [i for i in const if i[15] == STA]

    pN = [const[0][0]]
    pE = [const[0][1]]
    pU = [const[0][2]]
    pN = [-1 * i for i in pN]
    pE = [-1 * i for i in pE]
    pU = [-1 * i for i in pU]
    # pN.append(0)
    # pE.append(0)
    # pU.append(0)

    # print "pN: %s" % p
    # print "pE: %s" % pE
    # print "pU: %s" % pU

    pb = [[0, 0], [0, 0], [0, 0]]

    # pb[0], success = optimize.leastsq(errfunc, pN[:], args=(yearf-yearf[0], data[0]))
    # pb[1], success = optimize.leastsq(errfunc, pE[:], args=(yearf-yearf[0], data[1]))
    # pb[2], success = optimize.leastsq(errfunc, pU[:], args=(yearf-yearf[0], data[2]))
    pb[0], success = optimize.leastsq(errfuncl, pb[0], args=(yearf, data[0]))
    pb[1], success = optimize.leastsq(errfuncl, pb[1], args=(yearf, data[1]))
    pb[2], success = optimize.leastsq(errfuncl, pb[2], args=(yearf, data[2]))

    return pN, pE, pU, pb


def pvel(pl, pcov):
    """ """
    vunc = [None, None, None]
    vel = [None, None, None]

    for i in range(3):
        vel[i] = pl[i][1]
        vunc[i] = np.sqrt(np.diag(pcov[i]))[1]
        # print("{0:0.2f} {1:0.2f}".format( pl[i][1],vunc[i]) )

    return vel, vunc


def printvelocity(sta, ll, vel, vfile, pheader=False):
    """ """
    import os

    header = (
        "#lon       lat\t\t   N[mm]   DN[mm]  E[mm]  DE[mm]  U[mm]  DU[mm]\t\tStation"
    )

    if pheader is True:
        print(header, file=vfile)

    gpsvelo = "{0:5.6f} {1:5.6f}\t{2:7.2f} {5:7.2f}\t{3:7.2f} {6:7.2f}\t{4:7.2f} {7:7.2f}\t\t{8:s}".format(
        ll[1], ll[0], vel[0], vel[1], vel[2], vel[3], vel[4], vel[5], sta
    )

    print(gpsvelo, file=vfile)

    return gpsvelo


def detrend(
    x,
    y,
    Dy=None,
    fitfunc=lineperiodic,
    p=None,
    pcov=None,
    STA=None,
    onlyPeriodic=True,
    zref=False,
):
    """
    Returning detrend parameters very preliminary construction
    """

    import numpy as np

    # import dread.gread as gtf
    import geo_dataread.gps_read as gdrgps

    if Dy is None:
        Dy = np.ones(y.shape)

    # Handling parameters
    if p is not None:
        pass
    else:
        if STA:
            const = getDetrFit(STA, onlyPeriodic=onlyPeriodic)
            p0 = convconst(const)
        else:
            p0 = [None, None, None]

        p, pcov = fittimes(fitfunc, x, y, Dy, p0=p0)

    for i in range(3):
        y[i] = y[i] - fitfunc(x, *p[i])

    if zref:
        _, y, _ = gdrgps.vshift(x, y, Dy, uncert=20.0, refdate=None, Period=5)

    return y


def dPeriod(yearf, data, Ddata, startyear=None, endyear=None):
    """
    update( dict( [ [ line.split(',')[0], line.split(',')[1:] ] for line in args.eventf.read().splitlines() ] ) )

    returns a subperiod of input data (default behaviour: do nothing)

    Input:
        yearf: time array
        data: data array
        Ddata: same form as data
        startyear: default=None
        endyear: default=None

    Output:
        returns yearf, data, Data within the period defined by startyear and endyear

    """
    if startyear:
        index = np.where(yearf <= startyear - 0.001)
        yearf = np.delete(yearf, index)
        data = np.delete(data, index, 1)
        Ddata = np.delete(Ddata, index, 1)

    if endyear:
        index = np.where(yearf >= endyear + 0.001)
        yearf = np.delete(yearf, index)
        data = np.delete(data, index, 1)
        Ddata = np.delete(Ddata, index, 1)

    return yearf, data, Ddata


def vshift(yearf, data, Ddata, uncert=20.0, refdate=None, Period=5, offset=None):
    """
    Shifts time series data by the average value of the interval defined by
    reday and the number of days specified ()
    """

    from gtimes.timefunc import currYearfDate

    # Filtering a little, removing big outliers
    with np.errstate(invalid="ignore"):
        filt = Ddata < uncert
    filt = np.logical_and(np.logical_and(filt[0, :], filt[1, :]), filt[2, :])

    yearf = yearf[filt]
    data = np.reshape(data[np.array([filt, filt, filt])], (3, -1))
    Ddata = np.reshape(Ddata[np.array([filt, filt, filt])], (3, -1))

    if data.any():
        if not (offset is None):
            pass
        else:
            offset = estimate_offset(yearf, data, Ddata, refdate=refdate, Period=Period)

    data = np.array([data[i, :] - offset[i] for i in range(3)])

    return yearf, data, Ddata, offset


def estimate_offset(yearf, data, Ddata, refdate=None, Period=5):
    """
    Estimating offset of a time series at a reference (refdate) point for a given interval (Period)
    defaults at 5 days at the start of the time series
    """
    # averaging the first period days
    if refdate:
        startdate = currYearfDate(0, refdate)
        enddate = currYearfDate(Period, refdate)
        if Period < 0:
            tmpyearf, tmpdata, tmpDdata = dPeriod(
                yearf, data, Ddata, enddate, startdate
            )
        else:
            tmpyearf, tmpdata, tmpDdata = dPeriod(
                yearf, data, Ddata, startdate, enddate
            )

        if tmpdata.any():
            # if there are any data from this period
            offset = np.average(tmpdata[0:3, :], 1, weights=1 / tmpDdata[0:3, :])
        else:
            # We need to extrapolate
            # þarf að díla við þetta með því að módelera.
            offset = np.average(data[0:3, 0:j], 1, weights=1 / Ddata[0:3, 0:7])
    else:
        offset = np.average(data[0:3, 0:Period], 1, weights=1 / Ddata[0:3, 0:Period])

    return offset


def iprep(yearf, data, Ddata, uncert=20.0, offset=None):
    """
    Wrapper for vshift intendet for initializing the time series.
    converts to mm and initializes the start of the time series to zero
    """

    # converting to mm
    data *= 1000
    Ddata *= 1000
    return vshift(yearf, data, Ddata, uncert=uncert, offset=offset)


def filt_outl(yearf, data, Ddata, pb, errfunc, outlier):
    # Removing big outliers
    for i in range(3):
        index = np.where(abs(errfunc(pb[i], yearf - yearf[0], data[i])) > outlier[i])
        yearf = np.delete(yearf, index)
        data = np.delete(data, index, 1)
        Ddata = np.delete(Ddata, index, 1)

    return yearf, data, Ddata


def gamittoNEU(sta, mm=False, ref="plate", dstring=None):
    """
    convert a gamit time series to a single np.array with readable time tag
    """

    import geofunc.geofunc as gf

    yearf, data, Ddata = openGlobkTimes(sta)
    yearf, data, Ddata, _ = vshift(
        yearf, data, Ddata, uncert=1.1, refdate=None, Period=5, offset=None
    )

    # remove plata velocity
    if ref == "plate":
        plateVel = gf.plateVelo([sta])
        data[0, :] = data[0, :] - plateVel[0, 1] * (yearf - yearf[0])
        data[1, :] = data[1, :] - plateVel[0, 0] * (yearf - yearf[0])

    # convert to mm
    if mm:
        data = data * 1000
        Ddata = Ddata * 1000

    return gtoNEU(yearf, data, Ddata, dstring=dstring)


def gtoNEU(yearf, data, Ddata, dstring=None):
    """ """

    import numpy as np

    from gtimes.timefunc import convfromYearf

    if dstring == "yearf":  # use the decimal year format
        NEUdata = np.array(
            zip(yearf, data[0], data[1], data[2], Ddata[0], Ddata[1], Ddata[2]),
            dtype=[
                ("yearf", float),
                ("data[0]", float),
                ("data[1]", float),
                ("data[2]", float),
                ("Ddata[0]", float),
                ("Ddata[1]", float),
                ("Ddata[2]", float),
            ],
        )
    else:
        yearf = convfromYearf(yearf, dstring)

        NEUdata = np.array(
            zip(yearf, data[0], data[1], data[2], Ddata[0], Ddata[1], Ddata[2]),
            dtype=[
                ("yearf", "S23"),
                ("data[0]", float),
                ("data[1]", float),
                ("data[2]", float),
                ("Ddata[0]", float),
                ("Ddata[1]", float),
                ("Ddata[2]", float),
            ],
        )

    return NEUdata


def read_gps_data(
    sta,
    Dir=None,
    start=None,
    end=None,
    ref="plate",
    detrend_periodic=True,
    detrend_line=True,
    fit=False,
    detrend_period=[None, None],
    useSTA=None,
    useFIT=None,
    uncert=20.0,
    database=False,
    logging_level=logging.WARNING,
):
    """

    Expample usage:
        Return undetrended data in ITRF2008 reference frame:
            VMEY_raw, _ = gpsr.read_gps_data("VMEY", detrend_line=False, detrend_periodic=False, ref="itrf2008")
        Return undetrended, plate removed data:
            VMEY_plate, _ = gpsr.read_gps_data("VMEY", detrend_line=False, detrend_periodic=False, ref="plate")
        Return detrended data:
            VMEY_detrend, _ = gpsr.read_gps_data("VMEY", detrend_line=True, detrend_periodic=True, ref="plate")

    returns:
        data: dataframe with GPS data
        const: paramers for detrending the station
    """

    import logging
    import numpy as np
    from gtimes.timefunc import TimetoYearf

    # Handling logging
    logging.basicConfig(
        format="%(asctime)-15s [%(levelname)s] %(funcName)s: %(message)s",
        level=logging_level,
    )
    logging.getLogger().setLevel(logging_level)
    module_logger = logging.getLogger()
    if database:
        module_logger("calling gps data from a database")
        pass
    else:
        yearf, data, Ddata, _ = getData(sta, Dir=Dir, ref=ref)

    syearf, sdata, sDdata = yearf, data, Ddata

    # NOTE: detrending needs to be revised.
    const = getDetrFit(sta, useSTA=useSTA, useFIT=useFIT)
    p0 = convconst(const)
    pb = p0.copy()

    module_logger.info("Fitting constants:\n {}".format(p0))
    if fit:
        pass
    elif not np.all([p0[i][1:].dtype is np.dtype('float64') if p0[i] is not None else False for i in range(3)]):
        module_logger.info("Setting fit to lineperiodic")
        fit = "lineperiodic"

    if useFIT == "periodic":
        module_logger.info(
            '"{}" parameters from {} used in {}'.format(
                const["Fit"].values[0], const["UseSTA"].values[0], sta
            )
        )
        module_logger.info('Setting the fit to a "line" for estimating the rate')
        fit = "line"

    syearf, sdata, sDdata = dPeriod(
        yearf,
        data,
        Ddata,
        startyear=detrend_period[0],
        endyear=detrend_period[1],
    )

    if fit:
        module_logger.info("Fitting =====================")
        if len(syearf) == 0:
            module_logger.warning(
                "No data for interval {}-{} try using the whole time series".format(
                    *detrend_period
                )
            )
            syearf, sdata, sDdata = yearf, data, Ddata

        module_logger.info("Fitting the data to a {}".format(fit))
        if fit == "line":
            if p0[0] is not None:
                pt = [p0[i][0:2] for i in range(3)]
            else:
                pt = p0
            module_logger.warning("initial parameters p0 {}".format(pt))
            pb, _ = fittimes(line, syearf, sdata, sDdata, p0=pt)
            module_logger.info("Estimated parameters for {} {}".format(sta, pb))
            if p0[0] is not None:
                pb = [np.concatenate((pb[i], p0[i][2:])) for i in range(3)]
            else:
                pb = [np.concatenate((pb[i], np.zeros(4))) for i in range(3)]
            module_logger.warning("estimated parameters pb {}".format(pb))
                

        elif fit == "periodic":
            pb, _ = fittimes(periodic, syearf, sdata, sDdata, p0=p0)

        elif fit == "lineperiodic":
            pb, _ = fittimes(lineperiodic, syearf, sdata, sDdata, p0=p0)

        if fit == "undetrend":
            pb = [np.zeros(6) for i in range(3)]
            syearf = [None, None]


        const = convconst(const, pb)
        const.loc[sta, ["Starttime", "Endtime"]] = [syearf[0], syearf[-1]]

        if fit == "undetrend":
            pb = [np.zeros(6) for i in range(3)]
            const = convconst(const, pb)
            const.loc[sta, ["Starttime", "Endtime"]] = [None, None]

    else:
        module_logger.info("Not fitting")

    if np.all([pb[i][1:].dtype is np.dtype('float64') if pb[i] is not None else False for i in range(3)]):
        if detrend_periodic:
            module_logger.info("Periodic detrending")
            module_logger.debug("Fitt parameters: {}".format(pb))
            data = detrend(yearf, data.copy(), Ddata, fitfunc=periodic, p=pb)

        if detrend_line:
            module_logger.info("linear detrending")
            pc = [pb[i][0:2] for i in range(3)]
            module_logger.debug("Fitt parameters: {}".format(pc))
            data = detrend(yearf, data.copy(), Ddata, fitfunc=line, p=pc)
    else:
        module_logger.warn("Fitt parameters are unknown pb={}".format(pb))
        if (detrend_periodic or detrend_line):
            module_logger.warning(
                "Will try to estimate the parameters based the whole dataset and detrend"
            )
            data = detrend(yearf, data.copy(), Ddata, fitfunc=lineperiodic)
    # ----------------------------------------------------

    if start:
        startf = TimetoYearf(start.year, start.month, start.day)
    else:
        startf = yearf[0]
    if end:
        endf = TimetoYearf(end.year, end.month, end.day)
    else:
        endf = yearf[-1]

    yearf, data, Ddata = dPeriod(yearf, data, Ddata, startyear=startf, endyear=endf)
    yearf, data, Ddata, _ = vshift(yearf, data, Ddata, uncert=uncert)

    data = convGlobktopandas(yearf, data, Ddata)

    data["hlength"] = np.sqrt(np.square(data[["east", "north"]]).sum(axis=1))
    data["hangle"] = np.rad2deg(np.arctan2(data["north"], data["east"]))
    data["Dhlength"] = np.sqrt(np.square(data[["Deast", "Dnorth"]]).sum(axis=1))

    module_logger.info("Dataframe columns:\n" + str(data.columns) + "\n")
    module_logger.info("Input time period: ({}, {})".format(start, end))
    module_logger.info("dataframe First and Last lines:\n" + str(data.iloc[[0, -1]]))
    module_logger.info("detrend parameter row:\n" + str(const))
    module_logger.debug("Dataframe shape: {}".format(str(data.shape)))
    module_logger.debug("dataframe types:\n" + str(data.dtypes) + "\n")

    return data, const


#
# Other functions
#


def toDateTime(yearf):
    """
    from floating point year to datetime
    """
    from gtimes.timefunc import TimefromYearf

    tmp = []

    for i in range(len(yearf)):
        tmp.append(TimefromYearf(yearf[i]))

    return tmp


def toord(yearf):
    """
    from floating point year to floating point ordinal
    """
    from gtimes.timefunc import TimefromYearf

    for i in range(len(yearf)):
        yearf[i] = TimefromYearf(yearf[i], "ordinalf")

    return yearf


def fromord(yearf):
    # from floating point year to floating point ordinal

    from gtimes.timefunc import TimetoYearf

    for i in range(len(yearf)):
        yearf[i] = Timeto(yearf[i], "ordinalf")

    return yearf


def getData(
    sta,
    fstart=None,
    fend=None,
    ref="itrf2008",
    Dir=None,
    tType="TOT",
    uncert=15,
    offset=None,
):
    """
    extracting and filtering data to prepeare for plotting
    """

    import geofunc.geofunc as gf

    if tType == "JOIN":
        tType = "TOT"

    yearf, data, Ddata = openGlobkTimes(sta, Dir=Dir, tType=tType)
    yearf, data, Ddata = dPeriod(yearf, data, Ddata, fstart, fend)
    if yearf is None or len(yearf) == 0:
        print("WARNING: no data for station {}".format(sta))
        return None, None, None, None
    yearf, data, Ddata, offset = iprep(yearf, data, Ddata, uncert=uncert, offset=offset)

    if offset is None:
        print("WARNING: offset determination failure for station {}".format(sta))

    if ref == "plate":
        plateVel = gf.plateVelo([sta])
        data[0, :] = data[0, :] - plateVel[0, 1] * 1000 * (yearf - yearf[0])
        data[1, :] = data[1, :] - plateVel[0, 0] * 1000 * (yearf - yearf[0])

    elif ref == "detrend":
        pN, pE, pU, pb = detrend(yearf, data, sta)
        pb_org = [pN, pE, pU]

        for i in range(3):
            data[i] = -errfunc(pb_org[i], yearf - yearf[0], data[i])

    elif ref == "itrf2008":
        pass

    else:
        plateVel = gf.plateVelo([sta], ref)
        data[0, :] = data[0, :] - plateVel[0, 1] * 1000 * (yearf - yearf[0])
        data[1, :] = data[1, :] - plateVel[0, 0] * 1000 * (yearf - yearf[0])

    return yearf, data, Ddata, offset


#
#   --- Private functions ---
#


def __converter(x):
    """
    The data extracted are converted to float and
    occational ******* in the data files needs to handled as NAN

    """
    import numpy as np

    try:
        return float(x)
    except:
        return np.nan

    # if x == '********':
    #    return np.nan
    # else:
    #    return float(x)



# ###############################
#
# checkPort.py 
#
# Code made by fjalar@vedur.is 
# Iceland Met Office
# 2014
#
# ###############################

######## IMPORT LIBRARIES ########

import socket, sys


        # NOTE: There is no PING check in this version of the module but should be part of it.
        #       The result is that less is known about the connection - is it the IP(router) that is not picking up
        #       or is it the receiver behind the port. 
        #       FUTURE DEVELOPER: look into paramiko or similar mouduels.

def isOpen(ip,receiver_port,DEBUG = False):

    #print "IP for checkPort: %s:%s" % (ip,port)
    currFunc=__name__+'.'+sys._getframe().f_code.co_name+' >>'    

    result = {'router':None,'receiver':None}
    router_port = '80'

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        if DEBUG: print "%s Checking IP: %s SOCKET: %s" % (currFunc,ip,receiver_port)
        s.settimeout(5)
        s.connect((ip, int(receiver_port)))
        s.settimeout(None)
        s.close()
        if DEBUG: print "%s Receiver success!" % currFunc
        result['router'] = True
        result['receiver'] = True

    except:
        if DEBUG: print "%s RECEIVER socket failed" % currFunc 
        result['receiver'] = False


    ## Althoug the socket to the receiver is down, the router may be up. Let's check that out.
    if result['receiver'] == False and ip != "localhost":
        if DEBUG: print "%s Cheking the Router next..." % currFunc

        s2 = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try: 
            if DEBUG: print "%s Checking IP: %s SOCKET: %s" % (currFunc,ip,router_port)
            s2.settimeout(3)
            s2.connect((ip, int(router_port)))
            s.settimeout(None)
            s2.close()
            if DEBUG: print "%s Router success!" % currFunc
            result['router'] = True

        except:
            if DEBUG: print "%s ROUTER socket failed" % currFunc 
            result['router'] = False

    # we have to handle 'localhost' and the ssh tunnels differently because the connection through them cannot be easily checked
    # if the equipment at the end of the tunnel does not answer. The answer given is simply N/N - not known.
    elif result['receiver'] == False and ip == 'localhost':
    
        result['router'] = 'N/N'
        
    
    return result



# ###############################
#
# netr.py 
# Code made by bgo@vedur.is
# Iceland Met Office
# 2011-2012
#
# ###############################


"""
Special module to handle Trimble 4700 receivers
In this program are the following functions:

1.  checkVolt(station_info)      - Open a ssh tunnel to a station.
2.  checkTemp(station_info)      - Open a ssh tunnel to a station.
3.  checkSession(station_info)   - Open a ssh tunnel to a station.
4.  checkLogging(station_info)   - Open a ssh tunnel to a station.
5.  commReport(station_info)
6.  download(station_info)       - Close a ssh tunnel to a station.

"""
######## IMPORT LIBRARIES ########

import sys, re, subprocess,copy
import utils.gpslib as gpslib
import utils.gpstime as gpstime
import utils.sshtc as sshtc
import trm4700

######## METHODS ########
def checkVolt(station,p_args,DEV,BAUD=9600,PAR="n",HTCR="--rtscts",state=None):
    
    trm = trm4700.trm4700(station,DEV)
    result=trm.getVolt()
    print result[value]
    return result[value]

def runStatus(station,p_args,DEV,BAUD=9600,PAR="n",HTCR="--rtscts",optionlist=("state",),path="/home/mogi/bin/rutils/242b/"):
    # call the rstatus command from rutils: by default -state is called, options awailable are  (svtracking,logging,station,session,state)

    currfunc=__name__+'.'+sys._getframe().f_code.co_name+' >>' # module.object name

    if p_args['debug']: print currfunc,'Starting...'
    if p_args['debug']: print currfunc,'Trimble 4700 Receivers'

    options=""
    for item in optionlist:
        options += " -"+item
    print options


    # Run the rutil CMD and return output and returncode
    serialcmd = "%srstatus -D%s -P%s -B%s %s%s" % (path,DEV,PAR,BAUD,HTCR,options)
    proc_checkvolt_returncode, proc_checkvolt_comm=gpslib.run_syscmd(serialcmd,p_args)

    status=gpslib.checkError(proc_checkvolt_returncode,proc_checkvolt_comm,1,p_args)


    if status == 0:
        value=proc_checkvolt_comm
    else:
        value=None

    return value

##

def runFile(station,p_args,DEV,BAUD=9600,PAR="n",HTCR="--rtscts",optionlist=("T",),path="/home/mogi/bin/rutils/242b/"):
    # call the rfile command from rutils: by default -T is called (lists files on receiver), options awailable are  (N,F,J,T,K"findex",-U"findex",-I"findex,local_filename")

    currfunc=__name__+'.'+sys._getframe().f_code.co_name+' >>' # module.object name

    if p_args['debug']: print currfunc,'Starting...'
    if p_args['debug']: print currfunc,'Trimble 4700 Receivers'

    options=""
    for item in optionlist:
        options += " -"+item
    print options


    # Run the curl CMD and return output and returncode
    serialcmd = "%srfile -D%s -P%s -B%s %s%s" % (path,DEV,PAR,BAUD,HTCR,options)
    proc_checkvolt_returncode, proc_checkvolt_comm=gpslib.run_syscmd(serialcmd,p_args)

    status=gpslib.checkError(proc_checkvolt_returncode,proc_checkvolt_comm,1,p_args)


    if status == 0:
        value=proc_checkvolt_comm
    else:
        value=None

    return value
def setSurvey(station,p_args,DEV):

    currfunc=__name__+'.'+sys._getframe().f_code.co_name+' >>' # module.object name

    if p_args['debug']: print currfunc,'Starting...'
    if p_args['debug']: print currfunc,'Trimble 4700 Receivers'

    print "Trimble 4700"    
    # Delete current survey
    serialcmd = "/home/mogi/bin/rutils/242b/rsurvey -D%s -B9600 -Pn --rtscts -E" % (DEV,)
    #serialcmd = "/home/mogi/python/trimble/gps/lapd/bin/242a/rsuvey -D/home/mogi/dev/%s -B9600 -Pn --rtscts -E" % (station['recv_id'])

    # Run the curl CMD and return output and returncode
    proc_checkvolt_returncode, proc_checkvolt_comm=gpslib.run_syscmd(serialcmd,p_args)

    # Start new survey
    serialcmd = "/home/mogi/bin/rutils/242b/rsurvey -D%s -B9600 -Pn --rtscts -Q -L15 -Z0 -K60" % (DEV)
    #serialcmd = "/home/mogi/python/trimble/gps/lapd/bin/242a/rsurvey -D/home/mogi/dev/%s -B9600 -Pn --rtscts -Q -L15 -K1440" % (station['recv_id'])
#    return integer

    # Run the curl CMD and return output and returncode
    proc_checkvolt_returncode, proc_checkvolt_comm=gpslib.run_syscmd(serialcmd,p_args)

    value=0
    if p_args['debug']: print  "%s value returned %s:" % (currfunc ,value)
    return value

